/***************************************************************************
                          animatedsprite.h  -  description
                             -------------------
    begin                : Mit Aug 21 2002
    copyright            : (C) 2002, 2004 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ANIMATEDSPRITE_H
#define ANIMATEDSPRITE_H

#include <qcanvas.h>
#include "tpbase.h"

#include "animation.h"

/**Used in conjunction with Animation-class. TODO: document, add sound handling
  *@author Thomas Friedrichsmeier
  */

class AnimatedSprite : public QCanvasSprite, public TPBase  {
public: 
/** Constructor. Note, that you will have to call setAnimation before calling tick () the first time, or the app will crash */
	AnimatedSprite();
/** Destructor. Does not delete the attached Animation. You'll have to do that by hand. */
	~AnimatedSprite();
/** Advances the animation (taking care of all the details, including sound). Returns true if this tick finishes the animation */
	bool tick ();
/** Set the direction to direction (i.e. AnimForward or AnimReverse) */
	void setDirection (AnimDirections direction);
/** Read current direction */
	AnimDirections getDirection () { return static_cast<AnimDirections> (_direction); };
/** Sets the Animation to use and resets settings for frame, repetition... also stops any sound that were still playing */
	void setAnimation (Animation *animation, AnimDirections direction = AnimForward);
/** Return a pointer to the animation currently set */
	Animation *getAnimation () { return anim; };
/** Sets the orientation (i.e. going left or right etc. See TPBase::AnimationOrientations) */
	void setOrientation (AnimOrientations orientation);
/** Read current orientation */
	AnimOrientations getOrientation () { return static_cast<AnimOrientations> (_orientation); };

/** Starts sound-processing. Since the sprite is never explicitely told, when it gets started to be used, this has to be called
	manually (but only once for the animation, even if it has several repetitions) */
	void startSound (float pitch);
/** Stops sound-processing. See above. Will also be called from setAnimation, so usually no need to call manually */
	void stopSound ();
private:
	float sound_pitch;
	bool sound_processing;
	int repetition;
	int frame;
	int _direction;
	int _orientation;
	Animation *anim;
};

#endif
