/***************************************************************************
                          animation.cpp  -  description
                             -------------------
    begin                : Mit Aug 21 2002
    copyright            : (C) 2002, 2004 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "animation.h"

#include "qdom.h"
#include "cdp.h"
#include "klocale.h"

// TODO: add support for text Pixmap(Arrays)
Animation::Animation(const QDomElement *description, const int options, const int soundoptions, QString suffix_a, QString suffix_b){
	readAnimation (description, suffix_a, suffix_b, options, soundoptions);
}

Animation::~Animation(){
	CachablePixmapArray::discard (seq[0]);
	CachablePixmapArray::discard (seq[1]);
}

void Animation::readAnimation (const QDomElement *descr, QString & sufa, QString & sufb, const int opts, const int soundopts) {

	if (descr->hasAttribute ( "text" )) {		// if it has text-attribute, create a text-"animation" and disregard everything else except sound
		frames = 1;
		seq[0] = cdp->createTextPixmapArray(*descr);
		seq[1] = CachablePixmapArray::copy (seq[0]);
		frame_period = cdp->get_int_attribute("text_period", *descr, 5, 1000000, 1000, Info);
		repetitions = 1;
		continuous = false;
		oscillating = false;
	} else {

		frames = cdp->get_int_attribute("frames", *descr, 1, 10000, 1, Info);

		bool two_patterns = false;
		QString file_pattern = "file_pattern";
		sufa = file_pattern + sufa;
		sufb = file_pattern + sufb;
		if (sufa != sufb) {
			if (descr->hasAttribute(sufa) && descr->hasAttribute(sufb)) {
				two_patterns = true;
			} else {
				if (descr->hasAttribute (sufb)) {			// in case sufa is invalid, but sufb is there: use sufb
					sufa = sufb;
// TODO: debug_message
				} else if (descr->hasAttribute (file_pattern)) {
					sufa = sufb = file_pattern;
// TODO: debug_message
				} else {
// TODO: debug_message
				}
			}
		}

		bool preserve_alpha = cdp->get_bool_attribute("alpha", *descr, false, Info);
		seq[0] = cdp->createPixmapArray(sufa, *descr, &frames, 20, 20, preserve_alpha);
		if (two_patterns) {
			seq[1] = cdp->createPixmapArray(sufb, *descr, &frames, 20, 20, preserve_alpha);
		} else {
			seq[1] = CachablePixmapArray::copy (seq[0]);
		}

		frame_period = cdp->get_int_attribute("anim_frame_period", *descr, 5, 10000, 50, Info);

	// read options
		if (opts & ReadAnimContinuous) {
			continuous = cdp->get_bool_attribute("anim_continuous", *descr, static_cast<bool> (opts & DefaultAnimContinuous), Info);
		} else {
			continuous = static_cast<bool> (opts & DefaultAnimContinuous);
		}
		if (opts & ReadAnimRepetitions) {
			if (descr->hasAttribute ("repetitions")) {
				if (continuous) {
					cdp->debug_msg (i18n ("'repetitions'-attribute given for continuous animation. Assuming animation is not continuous after all."), Warn, descr);
					continuous = false;
				}
				repetitions = cdp->get_int_attribute("repetitions", *descr, 1, 1000, 1, Info);
			}
		} else {
			repetitions = 1;
		}
		if (opts & ReadAnimOscillating) {
			oscillating = cdp->get_bool_attribute("anim_oscillating", *descr, static_cast<bool> (opts & DefaultAnimOscillating), Info);
		} else {
			oscillating = static_cast<bool> (opts & DefaultAnimOscillating);
		}
	}			// end if text-attribute

// read sound
	if (opts & ReadSound) {
		int useopts = soundopts;
		if (repetitions < 2) {
			useopts -= (soundopts & (DefaultRepetitious | ReadRepetitious));		// only a repetitious animation may have a continuous sound!
		}
		sound.create (*descr, useopts, Info);
	}
}
